const optionSetMapping = {};

function initializeOptionSetFiltering(optionSetFiltersRaw, hide = true) {

    optionSetFiltersRaw.forEach((osf) => {

        if (!optionSetMapping.hasOwnProperty(osf.primaryName)) {
            optionSetMapping[osf.primaryName] = {};
            optionSetMapping[osf.primaryName] = {
                "allChildren": []
            };
        }

        if (!optionSetMapping[osf.primaryName].hasOwnProperty(osf.primaryValue)) {
            optionSetMapping[osf.primaryName][osf.primaryValue] = {};
        }

        const currentPrimary = optionSetMapping[osf.primaryName][osf.primaryValue];

        if (!currentPrimary.hasOwnProperty(osf.secondaryName)) {
            currentPrimary[osf.secondaryName] = [];
        }

        if (optionSetMapping[osf.primaryName]["allChildren"].indexOf(osf.secondaryName) === -1 &&
            osf.secondaryName !== "") {
            optionSetMapping[osf.primaryName]["allChildren"].push(osf.secondaryName);
        }

        currentPrimary[osf.secondaryName].push(osf.secondaryValue);
    });

    const optionSetCache = {};

    for (var primaryFieldName in optionSetMapping) {

        let primaryField = $("#" + primaryFieldName);

        primaryField.change(() => filterOptionSet(primaryField, optionSetCache, hide));
    }
}

function filterOptionSet(primaryField, optionSetCache, hide) {
    const primaryFieldName = primaryField.attr("id");
    const primaryFieldValue = primaryField.val();

    let success = false;

    if (primaryFieldValue !== '' && primaryFieldValue !== null) {
        // Grab the list of valid filtered values from the mapping

        if (optionSetMapping[primaryFieldName].hasOwnProperty(primaryFieldValue)) {
            let childObjects = optionSetMapping[primaryFieldName].allChildren;
            if (childObjects !== undefined) {

                for (var i = 0; i < childObjects.length; i++) {

                    const childFieldName = childObjects[i];
                    const childField = $("#" + childFieldName);
                    const childFieldValues = optionSetMapping[primaryFieldName][primaryFieldValue][childFieldName] ?? [];

                    childField.val("").change();

                    if (!optionSetCache.hasOwnProperty(childFieldName)) {
                        cacheOptionSet(childFieldName, optionSetCache);
                    }

                    // Clear non-blank options from dropdown
                    clearDropdownOptions(childFieldName);

                    // For each of the found valid values, add the correct option if available
                    childFieldValues.forEach(function (osValue) {
                        let optionFilter = optionSetCache[childFieldName].filter(function () { return this.value == osValue });

                        if (optionFilter.length > 0) {
                            childField.append(optionFilter[0]);
                        }
                    });

                    if (childField.children().length > 1 && hide) {
                        childField.parents().eq(1).css("display", "block");

                        success = true;
                    }
                }
            }
        }
    }

    if (!success) {
        optionSetMapping[primaryFieldName].allChildren.forEach((child) => {

            const childField = $("#" + child);
            childField.val("").change();

            if (hide) {
                childField.parents().eq(1).hide();
            }
        });
    }
}

function cacheOptionSet(fieldName, optionSetCache) {
    optionSetCache[fieldName] = $("#" + fieldName + " option").attr("selected", false).filter(function () { return this.value !== '' });
}

function clearDropdownOptions(fieldName) {
    // Retrieve non-blank dropdown options and remove them
    // Don't worry about resetting the value as user will be redirected on select
    let dropdownOptions = $("#" + fieldName + " option").filter(function () { return this.value !== '' });
    dropdownOptions.remove();
}